const PLUGIN_NAME = 'examplePlugin';
const NODE_PATH = process.argv[0];
const DIST_PATH = './dist/' + PLUGIN_NAME;

//path to basket installation
const BASKET_PATH = '../../wbk/dist';

module.exports = function (grunt)
{
    "use strict";

    //setup tasks
    var defaultTasks = [
        "clean:dist",
        "webpack:main",
        "copy:dist",
        "replace:pluginVersion",
        "copy:toBasket",
        "exec:updatePlugins"
    ];

    //default/development build task
    grunt.registerTask("default", defaultTasks);

    //production build task
    grunt.registerTask("production", defaultTasks);

    //setup production environment
    let isProduction = grunt.cli.tasks.length > 0 && grunt.cli.tasks[0] === "production";
    if (isProduction) {
        process.env.NODE_ENV = "production";
    }

    //load package json
    const pkgJson = require('./package.json');

    //load webpack config
    const webpackConfig = require('./webpack.config');

    //setup grunt config
    const gruntConfig = {
        clean: {
            //task to clean the distribution folder
            dist: [
                DIST_PATH
            ]
        },

        copy: {
            //task to copy files from src to dist
            dist: {
                files: [
                    { cwd: "./src", src: "plugin.json", dest: DIST_PATH, expand: true },
                    { cwd: "./", src: "README.md", dest: DIST_PATH, expand: true },
                    { cwd: "./", src: "CHANGELOG.md", dest: DIST_PATH, expand: true }
                ]
            },
            //task to copy distribution folder to basket
            toBasket: {
                files: [
                    { cwd: DIST_PATH, src: "**/*.*", dest: BASKET_PATH + "/plugins/" + PLUGIN_NAME, expand: true }
                ]
            }
        },

        webpack: {
            options: {
                stats: false
            },
            //webpack build task
            main: webpackConfig
        },

        replace: {
            //task to replace plugin manifest version with the version from the package
            pluginVersion: {
                options: {
                    patterns: [
                        {
                            match: "PLUGIN_VERSION",
                            replacement: pkgJson.version
                        }
                    ]
                },
                files: [
                    { cwd: DIST_PATH, src: "plugin.json", dest: DIST_PATH, expand: true }
                ]
            }
        },

        exec: {
            //task to update the baskets plugin registry
            updatePlugins: {
                cwd: BASKET_PATH + "/plugins",
                cmd: function () { return `"${NODE_PATH}" updatePlugins.js` }
            }
        }
    };


    //init grunt with config
    grunt.initConfig(gruntConfig);

    //load grunt tasks
    grunt.loadNpmTasks('grunt-webpack');
    grunt.loadNpmTasks('grunt-contrib-clean');
    grunt.loadNpmTasks("grunt-contrib-copy");
    grunt.loadNpmTasks("grunt-exec");
    grunt.loadNpmTasks("grunt-replace");
};
