import * as React from "react";

/** predefined message box icons */
export type MessageBoxIcon = "Information" | "Question" | "Warning" | "Error";

export interface MessageBoxOptions
{
    title: string;
    message: string;
    messageIsPreformatted?: boolean;

    /**
     * Optional custom content.
     * Can be an html string or a function returning a ReactNode.
     */
    customContent?: string | (() => React.ReactNode);

    /** optional preformatted details */
    detailMessage?: string;

    /**
     * Array of buttons for the message box.
     * A custom text or a predefined text can be used e.g. `egr.wbkApp.commonResources.buttonOk`
    */
    buttons: Array<string>;

    /**
     * Optional predefined icon.
     */
    icon?: MessageBoxIcon;

    /**
     * Optional custom icon defined by an image url.
     */
    iconUrl?: string;

    /**
     * Optional icon defined by name.
     */
    iconName?: string;

    /** if true the message box can be closed (default: true) */
    canClose?: boolean;

    /** if true the message can be copied to clipboard (default: false) */
    canCopy?: boolean;

    /** if true the message can be reported as an error (default: false) */
    canReportAsError?: boolean;

    /**
     * If true the message box will be displayed with high priority and will overlay even
     * the global loading indicator. (default: false)
     */
    isSignificant?: boolean;
}