import { EaiwsSession } from "@egr/wcf/modules/eaiws";
import { ListenableEvent } from "@egr/wcf/modules/utils";

export type UserRestriction =
    /** Editing of project informations and items is not allowed. (read-only access) */
    "edit" |

    /** Editing the project informations is not allowed. (read-only access) */
    "project.edit" |

    /** The project calculation will not be available. */
    "project.calculation" |
    /** Editing the project calculation is not allowed. (read-only access) */
    "project.calculation.edit" |
    /** The project purchase calculation will not be available. */
    "project.calculation.purchaseCalculation" |
    /** The condition type will not be visible in the project calculation. */
    "project.calculation.conditionType" |
    /** The margin will not be visible in the project calculation. */
    "project.calculation.margin" |

    /** The project customer information will not be available. */
    "project.customer" |
    /** Editing the project customer information is not allowed. (read-only access) */
    "project.customer.edit" |
    /** Editing/switching the customer by using the customer search is not allowed. (read-only access) */
    "project.customer.search.edit" |
    /** Editing the general customer data is not allowed. (read-only access) */
    "project.customer.general.edit" |
    "project.customer.general.company.edit" |
    "project.customer.general.externalCustomerId.edit" |
    "project.customer.general.customerNumber.edit" |
    "project.customer.general.generalAgreementNumber.edit" |
    "project.customer.general.customerRelatedRemarks.edit" |

    /** The customer main address will not be available. */
    "project.customer.mainAddress" |
    /** Editing the customer main address is not allowed. (read-only access) */
    "project.customer.mainAddress.edit" |
    /** The customer shipping address will not be available. */
    "project.customer.shippingAddress" |
    /** Editing the customer shipping address is not allowed. (read-only access) */
    "project.customer.shippingAddress.edit" |
    /** The customer billing address will not be available. */
    "project.customer.billingAddress"|
    /** Editing the customer billing address is not allowed. (read-only access) */
    "project.customer.billingAddress.edit" |

    /** The project data will not be available. */
    "project.data" |
    "project.data.projectName" |
    "project.data.projectNumber" |
    "project.data.projectState" |
    "project.data.projectDate" |
    "project.data.projectValidToDate" |
    "project.data.externalReferenceNumber" |
    "project.data.externalReferenceText" |
    "project.data.description" |
    "project.data.keywords" |
    "project.data.dataLanguage" |
    "project.data.defaultPriceDate" |
    "project.data.headerText" |
    "project.data.footerText" |

    /** Text formatting functions for project texts will not be available. */
    "project.data.textFormatting" |
    /** Editing the project data is not allowed. (read-only access) */
    "project.data.edit" |
    "project.data.projectName.edit" |
    "project.data.projectNumber.edit" |
    "project.data.projectState.edit" |
    "project.data.projectDate.edit" |
    "project.data.projectValidToDate.edit" |
    "project.data.externalReferenceNumber.edit" |
    "project.data.externalReferenceText.edit" |
    "project.data.description.edit" |
    "project.data.keywords.edit" |
    "project.data.dataLanguage.edit" |
    "project.data.defaultPriceDate.edit" |
    "project.data.headerText.edit" |
    "project.data.footerText.edit" |

    /** The project party in charge information will not be available. */
    "project.partyInCharge" |
    /** Editing the project party in charge information is not allowed. (read-only access) */
    "project.partyInCharge.edit" |

    /** The project currency and taxes will not be available. */
    "project.tax" |
    /** Editing currency and taxes is not allowed. (read-only access) */
    "project.tax.edit" |
    /** Currency exchange rates will not be available. */
    "project.tax.exchangeRates" |
    /** Editing the currency exchange rates is not allowed. (read-only access) */
    "project.tax.exchangeRates.edit" |
    /** Tax rates will not be available. */
    "project.tax.taxRates" |
    /** Editing the tax rates is not allowed. (read-only access) */
    "project.tax.taxRates.edit" |

    /** The project terms will not be available. */
    "project.terms" |
    /** Editing the project terms is not allowed. (read-only access) */
    "project.terms.edit" |

    /** Editing or creating items is not allowed. (read-only access) */
    "item.edit" |

    /** Additional user defined item images will not be available. */
    "item.additionalImages" |

    /** Extra item texts (e.g. special model information) will not be available. */
    "item.extraTexts" |
    /** Editing the extra item texts is not allowed. (read-only access) */
    "item.extraTexts.edit" |

    /** The external item number of items will not be available. */
    "item.externalItemNumber" |
    /** Editing of the external item number of items is not allowed. (read-only access) */
    "item.externalItemNumber.edit" |

    /** The item comments will not be available. */
    "item.comments" |
    /** Editing of item comments item is not allowed. (read-only access) */
    "item.comments.edit" |

    /** The item calculation will not be available. */
    "item.calculation" |
    /** Editing the item calculation is not allowed. (read-only access) */
    "item.calculation.edit" |
    /** The item purchase calculation will not be available. */
    "item.calculation.purchaseCalculation" |
    /** The condition type will not be visible in the item calculation. */
    "item.calculation.conditionType" |
    /** The margin will not be visible in the item calculation. */
    "item.calculation.margin" |
    /** The `update condition amounts` function will not be available. */
    "item.calculation.updateConditionAmounts" |

    /** The item configuration will not be available. */
    "item.configuration" |
    /** Configuration of items/articles is not allowed. (read-only access) */
    "item.configuration.edit" |

    /** Creating folders will not be available. */
    "item.folder.create" |
    /** Folder sub totals will not be available. */
    "item.folder.subTotal" |

    /** Creating set articles will not be available. */
    "item.setArticle.create" |
    /** Converting a folder into a set article will not be available. */
    "item.setArticle.convertTo" |
    /** Breaking up a set article will not be available. */
    "item.setArticle.breakUp" |

    /** Creating text items will not be available. */
    "item.textItem.create" |

    /** User defined description for OFML articles will not be available. */
    "item.ofmlArticle.userDescription" |
    /** Editing the user defined description for OFML articles is not allowed. (read-only access) */
    "item.ofmlArticle.userDescription.edit" |
    /** Updating/migrating ofml articles will not be available. */
    "item.ofmlArticle.update" |

    /** Creating user articles will not be available. */
    "item.userArticle.create" |
    /** Converting an article into a user article will not be available. */
    "item.userArticle.convertTo" |
    /**
     * Editing the purchase price of a user article is not allowed. (read-only access)
     * The purchase price will be linked to the sales price in this case.
     */
    "item.userArticle.purchasePrice.edit" |

    /** Merging articles will not be available. */
    "item.mergeArticles" |

    /** Mark items as alternative (or vice versa) will not be available. */
    "item.markAsAlternative" |

    /** Mark items as optional (or vice versa) will not be available. */
    "item.markAsOptional" |

    /** Reporting will not be available. */
    "report" |

    /** Editing the report is not allowed. (read-only access) */
    "report.edit" |

    /** Order information / placing an order will not be available. */
    "order" |

    /** Editing the order information is not allowed. (read-only access) */
    "order.edit" |

    /** Editing the order number is not allowed. (read-only access) */
    "order.orderNumber.edit" |

    /** Editing the order number is not allowed. (read-only access) */
    "order.vendorNumber.edit" |

    /** Editing the order main address is not allowed. (read-only access) */
    "order.mainAddress.edit" |

    /** Editing the order shipping address is not allowed. (read-only access) */
    "order.shippingAddress.edit" |

    /** Editing the order billing address is not allowed. (read-only access) */
    "order.billingAddress.edit";

export interface UserRestrictionsProvider
{
    /** Set of current user restrictions. */
    readonly restrictions: ReadonlySet<UserRestriction> | null;

    /** Will be triggered every time the `restrictions` have changed. */
    readonly eventRestrictionsChanged: ListenableEvent<void, UserRestrictionsProvider>;
}

/**
 * Special user context for pCon.login scenarios.
 * @see User.context
 */
export interface PConLoginContext
{
    readonly contextType: "pCon.login";
    getToken(): Promise<string>;
}

export interface User
{
    /** Login name of the user. */
    readonly name: string;

    /** Optional full name of the user (e.g. first name + last name) */
    readonly fullName?: string;

    /** Optional url to the company logo. */
    readonly companyLogo?: string;

    /** Optional list of user roles */
    readonly roles?: ReadonlySet<string>;

    /** Optional key-value attributes for the user */
    readonly attributes?: ReadonlyMap<string, Array<string>>;

    /** Optional authentication context (e.g. a login token) */
    readonly context?: unknown;

    /** Optional url for user account management */
    readonly accountUrl?: string;

    /**
     * Optional user bound eaiws session.
     * This session is an authorized eaiws session and may provide user specific data and access rights.
     * It should be the preferred session for activities.
     * Note: This session should not be closed. Use `Utils.closeEaiwsSession()` to ensure this.
     */
    readonly eaiwsSession?: EaiwsSession;
}
