## projectOrder

This plugin implements an OEX export.

### Configuration

To configure the plugin the following section has to be added to the plugin settings section inside the applications `settings.json`.

```typescript
// Order Plugin
"projectOrder": {
    /**
     * Version of the OEX specification which should be used for the export. (default: "3.0.0")
     */
    "oexVersion": "2.3.0" | "3.0.0",
    /**
     * If true an OBX will be attached to the generated OEX. (default: true)
     */
    "attachOBX": boolean,
     /**
     * If true an PDF report will be attached to the generated OEX. (default: false)
     * @deprecated define a report template in application order settings instead
     */
    "attachPDF": boolean,
    /**
     * Report template which should be used for the attached PDF report. (default: "basket-standard-pro-order")
     * @deprecated define a report template in application order settings instead
     */
    "reportTemplate": string,
    /**
     * Path to store the order on the server using the v2 api of the EAIWS FileStorage plugin. 
     * If defined the order will be stored on the server. (default: undefined)
     */
    "fileStoragePath": string
    /**
     * Mail configuration name of the EAIWS Mail plugin. 
     * If defined (and `fileStoragePath` is undefined) the order will be 
     * delivered by email from the server. (default: undefined)
     */
    "mailConfig": string,
    /**
     * Fixed subject when sending oex by mail 
     */
    "mailSubject": string,
    /**
     * If true will append SalesOrganisation as a prefix to the file name.
     */
    "useSalesOrgAsFilePrefix": boolean,
    /**
     * If true order and quote number will be optional.
     */
    "isOrderNumberOptional": boolean,
    /**
     * If true vendor number will be optional.
     */
    "isVendorNumberOptional": boolean,
    /**
     * If true export images with OEX export 
     */
    "exportImages": boolean,
    /**
     * If true export invisible features. Default value is true.
     */
    "exportInvisibleFeatures": boolean,
    /**
     * Sets project username as header orgdata if present
     */
    "identicateUser": boolean
    /**
     * Name of the app which is stored inside the order xml. (default: pCon.basket Online) 
     */
    "appName": string
}
// Dealer Dialog
"projectExport-oexDealer": {
    /**
     * Version of the OEX specification which should be used for the export. (default: "3.0.0")
     */
    "oexVersion": "2.3.0" | "3.0.0",
    /**
     * If true an OBX will be attached to the generated OEX. (default: true)
     */
    "attachOBX": boolean,
     /**
     * If true an PDF report will be attached to the generated OEX. (default: false)
     */
    "attachPDF": boolean,
    /**
     * Report template which should be used for the attached PDF report. (default: "basket-standard-pro-order")
     */
    "reportTemplate": string,
    /**
     * Path to store the order on the server using the v2 api of the EAIWS FileStorage plugin. 
     * If defined the order will be stored on the server. (default: undefined)
     */
    "fileStoragePath": string
    /**
     * Mail configuration name of the EAIWS Mail plugin. 
     * If defined (and `fileStoragePath` is undefined) the order will be 
     * delivered by email from the server. (default: undefined)
     */
    "mailConfig": string,
    /**
     * If fales the order e-mail address is fixed and defined by the mailConfig. (default: true)
     */
    "dynamicOrderMail": boolean,
    /**
     * If true will append SalesOrganisation as a prefix to the file name.
     */
    "useSalesOrgAsFilePrefix": boolean,
    /**
     * If true order and quote number will be optional.
     */
    "isOrderNumberOptional": boolean,
    /**
     * If true vendor number will be optional.
     */
    "isVendorNumberOptional": boolean,
     /**
     * If false the manufacturer selection is disabled and all manufacturerss will be exported. (default: true)
     */
    "manufacturerSelectionEnabled": boolean,
     /**
     * Name of the user attribute of the logged in user which contains the client number.
     * If defined the client number will be initialized with the value if this attribute. 
     * Only supported when exporting all manufacturers.
     */
    "clientNoUserAttribute": string,
    /**
     * If true export invisible features. Default value is true.
     */
    "exportInvisibleFeatures": boolean,
    /**
     * Sets project username as header orgdata if present
     */
    "identicateUser": boolean,
    /**
     * Name of the app which is stored inside the order xml. (default: pCon.basket Online) 
     */
    "appName": string
}
```


### fileStoragePath

To configure the fileStoragePath the following tasks should be done:

 - make sure the file_storage plugin inside the EAIWS is installed
 - create a config file under etc/plugins/file_storage/v2_configs e.g. `02_orders.xml`
 - the config file must contain a number and a name
    - the number needs to be lower as the fallback file (99)
    - name must be identical to the value inside `fileStoragePath`
- inside the `02_orders.xml` the property `url_prefix` should also contain the identical name from `fileStoragepath`
- `data_path` targets the folder where the files are stored

Example of `02_orders.xml`

```xml
<?xml version="1.0" encoding="UTF-8" standalone="no"?>
<!DOCTYPE properties SYSTEM "http://java.sun.com/dtd/properties.dtd">
<properties>
  <entry key="allow_subdirectories">true</entry>
  <entry key="omit_url_prefix_from_data_path">true</entry>
  <entry key="url_prefix">orders/</entry>
  <entry key="data_path">/opt/pCon/EAIWS/etc/plugins/file_storage/data/v2/</entry>
</properties>
```
Example of `setting.json`

```typescript
// Order Plugin
"projectOrder": {
    /**
     * Path to store the order on the server using the v2 api of the EAIWS FileStorage plugin. 
     * If defined the order will be stored on the server. (default: undefined)
     */
    "fileStoragePath": "orders"
}
```

### mailConfig

To configure the mailConfig the following tasks should be done:

 - make sure the mail plugin inside the EAIWS is installed
 - create a config file under etc/plugins/mail/mail_config e.g. `test_order.xml`
 - the name inside `mailconfig` must be identical to the file name from the mail config

 Example of `test_order.xml`

```xml
<?xml version="1.0" encoding="UTF-8" standalone="no"?>
<!DOCTYPE properties SYSTEM "http://java.sun.com/dtd/properties.dtd">
<properties>
    <entry key="to">test@example.com</entry>
    <entry key="from">noreply@example.org</entry>
    <entry key="multipartMessages">true</entry>
</properties>
```

 Example of `setting.json`

```typescript
// Order Plugin
"projectOrder": {
    /**
     * Path to store the order on the server using the v2 api of the EAIWS FileStorage plugin. 
     * If defined the order will be stored on the server. (default: undefined)
     */
    "mailConfig": "test_order"
}
```