import { Plugin, PluginDescriptor, PluginCategory } from "./Plugin";

export interface PluginManager
{
    /** Returns the plugin descriptor for the given plugin name. */
    getDescriptor(pName: string): PluginDescriptor | null;

    /**
     * Returns the first available plugin descriptor for the given plugin category.
     * If multiple plugins exist for the same plugin category default plugins have lower priority.
     */
    getDescriptorByCategory(pCategory: PluginCategory, pSubCategory?: string): PluginDescriptor | null;

    /** Returns an array of all available plugin descriptors for the given plugin category. */
    getDescriptorsByCategory(pCategory: PluginCategory, pSubCategory?: string): Array<PluginDescriptor>;

    /** Retrieves a plugin by its name. The plugin will be loaded if it is not yet registered. */
    getPlugin(pName: string): Promise<Plugin>;

    /** Registers a new plugin instance. Has to be called from plugins after they were loaded. */
    registerPlugin(pPlugin: Plugin): void;

    /**
     * Disables a set of plugins (plugin names). Once a plugin is disabled it can not be activated again.
     * Disabling an already active plugin instance/component is not possible.
     */
    disablePlugins(pDisabledPlugins: ReadonlySet<string>): void;

    /**
     * Disables a set of plugin categories. Once a plugin category is disabled it can not be activated again.
     * Disabling an already active plugin instance/component is not possible.
     */
    disableCategories(pDisabledCategories: ReadonlySet<PluginCategory>): void;
}