import * as project from "@egr/wcf/modules/eaiws/project";

import { ProjectData, ProjectDataPlugin, ProjectDataOptions } from "./ProjectData";
import { UserRestriction } from "../types/User";

export type AddressFields = Exclude<keyof project.AddressData, "commAddresses" | "contacts" | "addressType">;
export type ContactFields = Exclude<keyof project.ContactData, "commAddresses"> | project.CommAddrType;

export interface AddressFormOptions extends ProjectDataOptions
{
    /** Address type which will be edited with this address form. */
    addressType: project.AddressType;

    /** CSS padding of this form. Can be used to override the default padding. */
    padding?: string;

    /** List of address fields which will be marked as required. */
    requiredAddressFields?: ReadonlyArray<AddressFields>;

    /** List of contact fields which will be marked as required. */
    requiredContactFields?: ReadonlyArray<ContactFields>;

    /** Restriction which will used for the editability check of the address fields. (default: `project.edit`) */
    addressEditRestriction?: UserRestriction;

    /** Restriction which will used for the editability check of the contact fields. (default: `project.edit`) */
    contactEditRestriction?: UserRestriction;

    /**
     * BeginBusySection and endBusySection can be defined to show a custom loading animation.
     * If not defined a default animation will be shown.
     */
    beginBusySection?(): void;

    /**
     * BeginBusySection and endBusySection can be defined to show a custom loading animation.
     * If not defined a default animation will be shown.
     */
    endBusySection?(): void;
}

/**
 * The AddressForm component is a field group (Form) that collectively represents an address.
 * Fields include name, street, postcode etc.
 */
export interface AddressForm extends ProjectData
{
}

/**
 * AddressFormPlugin is used to create an AddressForm plugin component. An AddressForm displays address information.
 * This is reused accross the application for any eaiws.project.AddressType.
 * - Plugin category = `AddressForm`
 */
export interface AddressFormPlugin extends ProjectDataPlugin<AddressForm, AddressFormOptions>
{
}
