import { ActivityPlugin, Activity } from "./Activity";
import { User, UserRestriction } from "../types/User";

/**
 * Integration activity.
 * createComponent has to return an instance of Integration.
 * - Plugin category = `Integration`
 */
export interface IntegrationPlugin extends ActivityPlugin
{
}

export interface ButtonOptions
{
    /** custom tooltip for the button */
    tooltip?: string;
    /**
     * custom icon name for the button (ignored if `iconUrl` is specified)
     * See `https://materialdesignicons.com/` for list of icons.
     * Icon names have to be prefixed with `mdi-`.
     */
    iconName?: string;
    /** custom icon url for the button */
    iconUrl?: string;
}

export interface Integration extends Activity
{
    /** Current active user. */
    readonly user: User | null;

    /** If true and `user` is defined the application will show the user information. */
    readonly showUser: boolean;

    /**
     * If true the application will show a `Done` button which can be used
     * from the user to finish the integration .
     */
    readonly showDoneButton: boolean;

    /** Additional options for the done button. */
    readonly doneButtonOptions?: ButtonOptions;

    /**
     * Returns a set of restrictions for the current user. If null the
     * default restrictions of the application will be used.
     */
    readonly userRestrictions: ReadonlySet<UserRestriction> | null;
}

