import { BasketItem } from "@egr/wcf/modules/bsk";
import { ListenableEvent } from "@egr/wcf/modules/utils";

import { Plugin } from "../Plugin";
import { ProjectEditor } from "./ProjectEditor";

/**
 * Available sub categories for `ItemInfoExtension` plugins
 */
export type ExtensionSubCategory = "InfoProvider";

/**
 * Type for informations.
 *  - `string` single line value
 *  - `text` multi line value
 *  - `quoteText` multi line value which provides additional quote related information (currently only used for ofml articles)
 */
export type InformationType = "string" | "text" | "quoteText";

export interface Information
{
    /** unique key of the information */
    key: string;
    /** dispalyed name of the information */
    name: string;
    /** Type of the information */
    type: InformationType;
    /** dispalyed value of the information */
    value: string;
}

/**
 * Extension for the item info which is used to display additional item information
 * - Plugin category = `ItemInfoExtension`
 * - Plugin sub category = `InfoProvider`
 */
export interface InfoProvider extends Plugin
{
    /**
     * Optional event which can be implemented to trigger an update if the information for an item has changed.
     */
    readonly eventItemInfoChanged?: ListenableEvent<BasketItem, InfoProvider>;

    /**
     * Returns the available information types for the given item.
     * Note: `pItem.getCachedItemProperies()` can be used to get the item properties.
     */
    getItemInfoTypes(pItem: BasketItem): Array<InformationType> | null;

    /** Will be called to retrieve the additional information. */
    getItemInfo(pItem: BasketItem, pEditor: ProjectEditor): Promise<Array<Information>>;
}


