import { EaiwsSession } from "@egr/wcf/modules/eaiws";
import * as project from "@egr/wcf/modules/eaiws/project";
import { ListenableEvent } from "@egr/wcf/modules/utils";

import { MenuAction } from "../Action";
import { Plugin } from "../Plugin";

/**
 * Available sub categories for `ProjectCustomerPickerExtension` plugins
 */
export type ExtensionSubCategory = "CustomerProvider";

export interface CustomerInfo
{
    company?: string;
    externalCustomerId?: string;
    customerNumber?: string;
    generalAgreementNumber?: string;
    customerRelatedRemarks?: string;
    addresses?: Array<project.AddressData>;
    customAppData?: Array<AppData>;
}

export interface AppData
{
    appKey: string;
    appData: Map<string, string>;
}

export interface FilterDefinition
{
    key: string;
    name: string;
    readonly: boolean;
    value?: string;
    options?: Map<string, string>;
}

/**
 * Extension for customer picker which is used as a data provider.
 * - Plugin category = `ProjectCustomerPickerExtension`
 * - Plugin sub category = `CustomerProvider`
 */
export interface CustomerProviderExtension extends Plugin
{
    /** Event which should be triggered if the filters have changed e.g. name or options have changed */
    eventFiltersChanged: ListenableEvent<void, CustomerProviderExtension>;

    /** Returns a list of available filters for this provider. */
    getFilters(pSession: EaiwsSession): Promise<Array<FilterDefinition> | null>;

    /**
     * Queries a list of customers using the provided filters.
     * @param pFilterValues Key -> value map for the query filters.
     */
    getCustomers(pFilterValues: ReadonlyMap<string, string>): Promise<Array<CustomerInfo>>;

    /**
     * Returns optional menu actions.
     */
    getMenuActions?(): Array<MenuAction>;
}